==> 0.0.10.tar.gz: resticbackup/resticbackup/s3.py <==
import os
import json
import argparse
import traceback
import subprocess
from shutil import which
from urllib.parse import urlparse

from boto3 import client as boto3_client
from boto3.session import Config as boto3_config


def scan_bucket(s3, s3_bucket, path):
    for page in s3.get_paginator("list_objects_v2").paginate(
        Bucket=s3_bucket,
        Delimiter="/",
        Prefix=path
    ):
        prefixes = set(i["Prefix"].split("/")[-2] for i in page.get("CommonPrefixes", []))
        contents = set(i["Key"].split("/")[-1] for i in page.get("Contents", []))

        # if it smells like a populated restic restic repo, return it and don't go deeper into it
        if "config" in contents and set(['data', 'index', 'keys', 'snapshots']).issubset(prefixes):
            yield path
        else:
            for prefix in prefixes:
                yield from scan_bucket(s3, s3_bucket, path + prefix + "/")


def discover_repos(s3, s3_bucket):
    """
    using the given s3 client, scan files in the bucket
    """
    yield from scan_bucket(s3, s3_bucket, "")


def prune(s3, restic_uri, username, password, repo_password, extra_args=None):
    print("\nPruning: " + restic_uri, "\n")

    env = dict(os.environ)
    env["AWS_ACCESS_KEY_ID"] = username
    env["AWS_SECRET_ACCESS_KEY"] = password
    env["RESTIC_PASSWORD"] = repo_password

    try:
        subprocess.check_call(
            [which("restic"), "-r", restic_uri, "prune"] + (extra_args or []),
            env=env
        )
    except subprocess.CalledProcessError:
        traceback.print_exc()
        print("assuming we didn't have the repo password, ignoring")


def main():
    parser = argparse.ArgumentParser(description="restic pruning wrapper")

    parser.add_argument("s3uri", help="s3 uri. Same format as restic, without the leading s3:")
    parser.add_argument("passwordsfile", help="file containing restic encryption passwords")
    parser.add_argument('args', nargs="*", help="additional arguments to pass to restic")

    args = parser.parse_args()
    uri = urlparse(args.s3uri)

    with open(args.passwordsfile) as f:
        passwords = json.load(f)

    default_password = passwords["default"]

    maybeport = ":{}".format(uri.port) if uri.port else ""
    s3_addr = "{}://{}{}".format(uri.scheme, uri.hostname, maybeport)
    s3_bucket = uri.path[1:]

    s3 = boto3_client(
        "s3",
        endpoint_url=s3_addr,
        aws_access_key_id=uri.username,
        aws_secret_access_key=uri.password,
        aws_session_token=None,
        config=boto3_config(signature_version="s3v4"),
        # verify=False,
    )

    for repo in discover_repos(s3, s3_bucket):
        restic_uri = "s3:{}/{}/{}".format(s3_addr, s3_bucket, repo)
        password = passwords.get("{}/{}".format(s3_bucket, repo)) or passwords.get(s3_bucket + "/") or default_password
        prune(s3, restic_uri, uri.username, uri.password, password, args.args)


if __name__ == "__main__":
    main()

==> 0.0.13.tar.gz: resticbackup/resticbackup/s3.py <==
import os
import json
import argparse
import traceback
import subprocess
from shutil import which
from urllib.parse import urlparse

from boto3 import client as boto3_client
from boto3.session import Config as boto3_config


def scan_bucket(s3, s3_bucket, path):
    for page in s3.get_paginator("list_objects_v2").paginate(
        Bucket=s3_bucket,
        Delimiter="/",
        Prefix=path
    ):
        prefixes = set(i["Prefix"].split("/")[-2] for i in page.get("CommonPrefixes", []))
        contents = set(i["Key"].split("/")[-1] for i in page.get("Contents", []))

        # if it smells like a populated restic restic repo, return it and don't go deeper into it
        if "config" in contents and set(['data', 'index', 'keys', 'snapshots']).issubset(prefixes):
            yield path
        else:
            for prefix in prefixes:
                yield from scan_bucket(s3, s3_bucket, path + prefix + "/")


def discover_repos(s3, s3_bucket):
    """
    using the given s3 client, scan files in the bucket
    """
    yield from scan_bucket(s3, s3_bucket, "")


def prune(s3, restic_uri, username, password, repo_password, extra_args=None):
    print("\nPruning: " + restic_uri, "\n")

    env = dict(os.environ)
    env["AWS_ACCESS_KEY_ID"] = username
    env["AWS_SECRET_ACCESS_KEY"] = password
    env["RESTIC_PASSWORD"] = repo_password

    try:
        subprocess.check_call(
            [which("restic"), "-r", restic_uri, "prune"] + (extra_args or []),
            env=env
        )
    except subprocess.CalledProcessError:
        traceback.print_exc()
        print("assuming we didn't have the repo password, ignoring")


def main():
    parser = argparse.ArgumentParser(description="restic pruning wrapper")

    parser.add_argument("s3uri", help="s3 uri. Same format as restic, without the leading s3:")
    parser.add_argument("passwordsfile", help="file containing restic encryption passwords")
    parser.add_argument('args', nargs="*", help="additional arguments to pass to restic")

    args = parser.parse_args()
    uri = urlparse(args.s3uri)

    with open(args.passwordsfile) as f:
        passwords = json.load(f)

    default_password = passwords.get("default")

    maybeport = ":{}".format(uri.port) if uri.port else ""
    s3_addr = "{}://{}{}".format(uri.scheme, uri.hostname, maybeport)
    s3_bucket = uri.path[1:]

    s3 = boto3_client(
        "s3",
        endpoint_url=s3_addr,
        aws_access_key_id=uri.username,
        aws_secret_access_key=uri.password,
        aws_session_token=None,
        config=boto3_config(signature_version="s3v4"),
        # verify=False,
    )

    for repo in discover_repos(s3, s3_bucket):
        restic_uri = "s3:{}/{}/{}".format(s3_addr, s3_bucket, repo)
        password = passwords.get("{}/{}".format(s3_bucket, repo)) or passwords.get(s3_bucket + "/") or default_password
        if not passwords:
            raise Exception("password not provided for: {}/{}".format(s3_bucket, repo))
        prune(s3, restic_uri, uri.username, uri.password, password, args.args)


if __name__ == "__main__":
    main()
